"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const assert = require("assert");
const path = require("path");
const fs = require("fs-extra");
const gaze = require("gaze");
const events = require("events");
const os = require("os");
const uuid = require("uuid/v4");
const webpack = require("./webpack");
const yarn = require("./yarn");
const server_1 = require("./server");
const jsonLines_1 = require("./jsonLines");
const node_fetch_1 = require("node-fetch");
const dependencyErrorRegex = /"(.*)#(.*)@(.*)" doesn't satisfy found match of "(.*)@(.*)"/;
const OUTDATED_CHECK_INTERVAL = 2 * 60 * 1000; // 2 minutes
process.on("unhandledRejection", (reason, p) => {
    console.log("Unhandled Rejection at: Promise", p, "reason:", reason);
    server_1.Server.logError(`Unhandled Rejection at: Promise ${p} reason: ${reason}`);
});
class Project extends events.EventEmitter {
    constructor(dir, staticPath, cachePath, author, token, api) {
        super();
        this.config = {
            paths: {
                code: "code",
                package: "package.json",
                metadata: "metadata",
            },
        };
        this._lastBuildOutput = null;
        this.sendScript = (socket) => {
            // If the compiled script is available, send that
            let emitScript;
            if (socket) {
                emitScript = script => socket.emit("script", script);
            }
            else if (this._server) {
                const server = this._server;
                emitScript = script => server.io.emit("script", script);
            }
            else {
                // Nothing to send the script to
                return;
            }
            let script;
            if (this._lastBuildOutput) {
                script = this._lastBuildOutput;
            }
            else if (fs.pathExistsSync(this.buildOutputPath)) {
                // If the script was not built yet, we send a cached version if available
                script = fs.readFileSync(this.buildOutputPath).toString();
            }
            else {
                // Send the initial script with Framer Library “preloaded”
                script = fs.readFileSync(path.join(__dirname, "../preload.js")).toString();
            }
            if (!script) {
                return;
            } // else
            emitScript(script);
        };
        this.buildStandalone = (destination, previewData) => __awaiter(this, void 0, void 0, function* () {
            assert(destination.startsWith("/"));
            assert(previewData);
            assert(this._lastBuildOutput);
            const build = this._lastBuildOutput;
            if (!build)
                return;
            function replace(template, placeholder, replacement) {
                // Neutralize occurrences of </script> that would mess up the page
                const scriptClose = "</script";
                let scriptCloseIndex = replacement.lastIndexOf(scriptClose);
                while (scriptCloseIndex >= 0) {
                    const prefix = replacement.substring(0, scriptCloseIndex);
                    const suffix = replacement.substring(scriptCloseIndex + placeholder.length);
                    // Resolves to <\/script, which in turn becomes </script if used within a JS string. For plaintext comments the escape doesn't matter because it's not in user-readable code. Note that we can't use CDATA because it's ignored in HTML5.
                    replacement = prefix + "<\\/script" + suffix;
                    scriptCloseIndex = replacement.lastIndexOf(scriptClose);
                }
                // String.replace isn't always reliable, either because of buffer limits or some sort of replacement string parsing
                // See https://github.com/framer/company/issues/10201
                const placeholderIndex = template.indexOf(placeholder);
                if (placeholderIndex >= 0) {
                    const prefix = template.substring(0, placeholderIndex);
                    const suffix = template.substring(placeholderIndex + placeholder.length);
                    template = prefix + replacement + suffix;
                }
                return template;
            }
            let template = fs.readFileSync(path.join(__dirname, "../../Vekter/standalone.html")).toString();
            template = replace(template, "__DATA__", previewData);
            template = replace(template, "__BUILD__", build);
            console.log("writing standalone output", this.standaloneOutputPath);
            const p1 = new Promise(resolve => {
                fs.writeFile(this.standaloneOutputPath, template, err => {
                    if (err)
                        console.error(err);
                    resolve();
                });
            });
            const p2 = copyResourceFiles(this.path, {
                "../../Vekter/scripts/react/react.development.js": "build/react.development.js",
                "../../Vekter/scripts/react-dom/react-dom.development.js": "build/react-dom.development.js",
                "../../Library/framer.js": "build/framer.js",
                "../../Vekter/styles/Draft.css": "build/Draft.css",
                "../../Vekter/styles/style.css": "build/style.css",
                "../../Vekter/standalone.js": "build/standalone.js",
                "../../Vekter/images/cursors/touch.png": "images/cursors/touch.png",
                "../../Vekter/images/cursors/touch@2x.png": "images/cursors/touch@2x.png",
                "../../Vekter/images/cursors/touch-active.png": "images/cursors/touch-active.png",
                "../../Vekter/images/cursors/touch-active@2x.png": "images/cursors/touch-active@2x.png",
            });
            yield p1;
            yield p2;
            yield fs.mkdirp(destination);
            yield fs.emptyDir(destination);
            let from = this.path;
            if (!from.endsWith("/") && from.length > 0) {
                from = from + "/";
            }
            const pathLength = from.length;
            const directories = [];
            const manifest = [];
            yield fs.copy(from, destination, {
                filter(src) {
                    let target = src.slice(pathLength);
                    if (target.length === 0)
                        return true;
                    if (target.startsWith("."))
                        return false;
                    if (path.basename(target).startsWith("."))
                        return false;
                    // note, we get passed in directories, and returning false will skip them completely
                    // and while we don't want node_modules, we do want store installed packages for their resources
                    if (target.startsWith("node_modules/") && !target.startsWith("node_modules/@framer")) {
                        return false;
                    }
                    if (target.match(/^code\/.*\.[tj]sx?/))
                        return false;
                    if (target === "build/index.js")
                        return false;
                    if (target === "design/document.json")
                        return false;
                    if (target === "package.json")
                        return false;
                    if (target === "README.md")
                        return false;
                    if (target === "tsconfig.json")
                        return false;
                    if (target === "yarn.lock")
                        return false;
                    if (fs.statSync(src).isDirectory()) {
                        directories.push(target + "/");
                    }
                    else {
                        manifest.push(target);
                    }
                    return true;
                },
            });
            // clear out any empty directories
            directories.forEach(dir => {
                if (manifest.find(file => file.startsWith(dir)))
                    return;
                fs.rmdirSync(path.join(destination, dir));
            });
            manifest.push("manifest.txt");
            fs.writeFileSync(path.join(destination, "manifest.txt"), manifest.join("\n") + "\n");
            console.log("standalone export done:", destination, manifest.length);
        });
        this.commandHandler = (command, info) => __awaiter(this, void 0, void 0, function* () {
            switch (command) {
                case server_1.ServerCommand.addPackage:
                    if (!info.packageName) {
                        console.warn("Missing package name, aborting installation");
                        break;
                    }
                    const isDevDependency = info.isDevDependency || false;
                    yield this.add(info.packageName, isDevDependency, info.context);
                    break;
                case server_1.ServerCommand.removePackage:
                    if (!info.packageName) {
                        console.warn("Missing package name, cannot remove");
                        break;
                    }
                    yield this.remove(info.packageName, info.context);
                    yield this.refreshOutdated();
                    break;
                case server_1.ServerCommand.publishPackage:
                    yield this.publish(info.version, info.packageName, info.displayName, info.context, info.access);
                    break;
                case server_1.ServerCommand.updatePackageInfo:
                    yield this.updatePackageInfo();
                    break;
                case server_1.ServerCommand.upgradeDependency:
                    const { packageName, version } = info;
                    if (!packageName || !version) {
                        return;
                    }
                    yield this.upgrade(packageName, version);
                    break;
                case server_1.ServerCommand.upgradeFramerLibrary:
                    // Note: simply running upgrade doesn't work: https://github.com/yarnpkg/yarn/issues/2127
                    yield this.upgradePeerAndDevDependencyToLatest("framer");
                    break;
                case server_1.ServerCommand.setIsDocumentDirty:
                    if (info.dirty !== undefined && this._server) {
                        this._server.setState({ isDocumentDirty: info.dirty });
                    }
                    break;
                case server_1.ServerCommand.reloadClients:
                    this.reloadClients();
                    break;
                case server_1.ServerCommand.updateCanvasData:
                    this.canvasData = info.canvasData;
                    if (!this._watcher)
                        return;
                    this._watcher.writeBuildOutput();
                    break;
                case server_1.ServerCommand.updateCanvasModule:
                    if (info.code)
                        fs.writeFileSync(this.canvasModulePath, info.code);
                    break;
            }
        });
        this._dir = path.resolve(dir);
        this._staticPath = staticPath ? path.resolve(staticPath) : undefined;
        this._cachePath = path.resolve(cachePath);
        this._token = token;
        this._api = api;
        this._outdatedInterval = setInterval(() => this.refreshOutdated(), OUTDATED_CHECK_INTERVAL);
        // Work around Yarn 1.7.0 bug, can be removed once https://github.com/yarnpkg/yarn/pull/5938 is merged and released.
        fs.mkdirpSync(path.join(this.yarnCachePath, "v1", "npm-@framer"));
        if (!fs.existsSync(this.path))
            throw Error(`path does not exist: ${this.path}`);
        // XXX HACK (Remove when identity is added)
        if (author && !this.packageJson.author) {
            this.updatePackageJson(packageJson => {
                packageJson.author = author;
                return packageJson;
            });
        }
        fs.ensureDirSync(this.metadataPath);
        this.watchAssets();
    }
    stop() {
        if (this._outdatedInterval) {
            clearInterval(this._outdatedInterval);
        }
        if (this._assetWatcher) {
            this._assetWatcher.close();
            this._assetWatcher = null;
        }
    }
    install() {
        return __awaiter(this, void 0, void 0, function* () {
            this._fixNodeModulesPath();
            // First check the local cache
            return this.runTask("install", yarn.install(["--prefer-offline", ...this.yarnOptions]).then(() => {
                this.reloadClients();
            }));
        });
    }
    add(packageName, isDevDependency, context = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            this._fixNodeModulesPath();
            const options = isDevDependency ? ["--dev"] : [];
            return this.runTask("add", yarn.add([packageName, ...this.yarnOptions, ...options]).then(() => {
                this.invalidateCompilerWatch();
                return this.updateServerPackagesListing();
            }), Object.assign({}, context, { packageName }));
        });
    }
    remove(packageName, context = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            this._fixNodeModulesPath();
            return this.runTask("remove", yarn.remove([packageName, ...this.yarnOptions]).then(() => {
                return this.updateServerPackagesListing();
            }), Object.assign({}, context, { packageName }));
        });
    }
    upgrade(packageName, packageVersion, context = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            this._fixNodeModulesPath();
            return this.runTask("upgrade", yarn.upgrade([`${packageName}@${packageVersion}`, ...this.yarnOptions]).then(() => {
                return this.updatePackageInfo();
            }), Object.assign({}, context, { packageName,
                packageVersion }));
        });
    }
    upgradePeerAndDevDependencyToLatest(packageName, context = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            this._fixNodeModulesPath();
            yield this.runTask("remove", yarn.remove([packageName, ...this.yarnOptions]), Object.assign({}, context, { packageName }));
            yield this.runTask("add", yarn.add([packageName, "--peer", "--latest", ...this.yarnOptions]), Object.assign({}, context, { packageName }));
            yield this.runTask("add", yarn.add([packageName, "--dev", "--latest", ...this.yarnOptions]), Object.assign({}, context, { packageName }));
            yield this.updatePackageInfo();
            this.reloadClients();
        });
    }
    publish(version, packageName, displayName, context = {}, access) {
        return __awaiter(this, void 0, void 0, function* () {
            // TODO: Should be done as part of the tasks?
            yield this.prepareForPublish(packageName, displayName);
            const additionalOptions = ["--no-git-tag-version"];
            if (version) {
                additionalOptions.push("--new-version", version);
            }
            if (access === "restricted" || access === "public") {
                additionalOptions.push("--access", access);
            }
            const taskPromise = this.buildDist().then(() => yarn.publish([...this.yarnOptions, ...additionalOptions]));
            return this.runTask("publish", taskPromise, Object.assign({}, context, { version }));
        });
    }
    getCheck() {
        return __awaiter(this, void 0, void 0, function* () {
            const checkOptions = [];
            return this.runTask("getCheck", yarn.check([...this.yarnOptions, ...checkOptions]));
        });
    }
    check() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this._server)
                return;
            const checkResult = yield this.getCheck();
            const jsonLines = jsonLines_1.JSONLines.parse(checkResult.error);
            const errors = jsonLines.filter(json => json.type === "error");
            if (errors.length === 0) {
                this._server.setState({ packageErrors: [] });
                return;
            }
            let packageErrors = [];
            for (const error of errors) {
                const message = error.data;
                if (/Found \d+ errors/.test(message)) {
                    continue;
                }
                const matches = message.match(dependencyErrorRegex);
                if (matches && matches.length === 6) {
                    let packageError = {
                        message: message,
                        name: matches[1],
                        dependency: {
                            packageName: matches[2],
                            versionRequirement: matches[3],
                            currentVersion: matches[5],
                        },
                    };
                    packageErrors.push(packageError);
                }
            }
            this._server.setState({ packageErrors });
        });
    }
    getOutdated() {
        return __awaiter(this, void 0, void 0, function* () {
            return this.runTask("getOutdated", yarn.outdated([...this.yarnOptions]));
        });
    }
    refreshOutdated() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this._server)
                return;
            const outdatedResult = yield this.getOutdated();
            const jsonLines = jsonLines_1.JSONLines.parse(outdatedResult.output);
            if (jsonLines.length === 0) {
                this._server.setState({ outdated: [] });
                return;
            }
            const tables = jsonLines.filter(json => json.type === "table");
            if (tables.length !== 1) {
                return Promise.reject("Can't find outdated info");
            }
            const info = tables[0].data.body;
            const outdated = info.map(server_1.OutdatedPackage.fromTableRow);
            this._server.setState({ outdated });
        });
    }
    updatePackageInfo() {
        return __awaiter(this, void 0, void 0, function* () {
            this.updateServerPackagesListing();
            yield this.refreshOutdated();
            yield this.check();
        });
    }
    build() {
        return __awaiter(this, void 0, void 0, function* () {
            // Used for CLI and testing.
            const compiler = new webpack.Compiler(this, {
                outputPath: this.buildOutputPath,
            });
            yield compiler.build();
            this._lastBuildOutput = compiler.buildOutput;
            return compiler;
        });
    }
    buildDist() {
        return __awaiter(this, void 0, void 0, function* () {
            const compiler = new webpack.Compiler(this, {
                excludeExternals: true,
                outputPath: this.distOutputPath,
            });
            yield compiler.build();
            return compiler;
        });
    }
    pack(version, packageName, displayName) {
        return __awaiter(this, void 0, void 0, function* () {
            // Used for testing.
            yield this.prepareForPublish(packageName, displayName);
            if (version) {
                yield yarn.run(["version", "--new-version", version, ...this.yarnOptions]);
            }
            const result = yield yarn.run(["pack", ...this.yarnOptions]);
            // Get the response from Yarn and parse the tarball path from it.
            const { data } = JSON.parse(result.output);
            if (typeof data !== "string")
                return null;
            // The path is only exposed through a formatted string, so extract it from there.
            const match = data.match(/^Wrote tarball to "(.*?)"\.$/);
            if (!match)
                return null;
            return match[1];
        });
    }
    // Use `context` to add extra info that the server should send along when reporting on this task
    runTask(name, task, context) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this._server) {
                return this._server.runAsTask(name, task, context);
            }
            else {
                // Just silently wait on the promise
                return task;
            }
        });
    }
    get token() {
        return this._token;
    }
    get api() {
        return this._api;
    }
    get staticPath() {
        return this._staticPath;
    }
    get yarnOptions() {
        return [
            "--cwd",
            this.path,
            "--skip-integrity-check",
            "--cache-folder",
            this.yarnCachePath,
            "--mutex",
            `file:${this.yarnMutexPath}`,
            "--json",
        ];
    }
    get yarnCachePath() {
        return path.join(this.cachePath, "yarn", "cache");
    }
    get yarnMutexPath() {
        return path.join(this.cachePath, "yarn", "mutex");
    }
    get path() {
        return this._dir;
    }
    get cachePath() {
        return this._cachePath;
    }
    get distOutputPath() {
        return path.join(this.path, "dist", "index.js");
    }
    get buildOutputPath() {
        return path.join(this.path, "build", "index.js");
    }
    get standaloneOutputPath() {
        return path.join(this.path, "index.html");
    }
    get nodeModulesPath() {
        return path.join(this.path, "node_modules");
    }
    get watcher() {
        return this._watcher;
    }
    get codePath() {
        return path.join(this.path, this.config.paths.code);
    }
    get canvasModulePath() {
        return path.join(this.codePath, "canvas.tsx");
    }
    get metadataPath() {
        return path.join(this.path, this.config.paths.metadata);
    }
    get packagePath() {
        return path.join(this.path, "package.json");
    }
    get packageJson() {
        try {
            return JSON.parse(fs.readFileSync(this.packagePath).toString());
        }
        catch (error) {
            return {};
        }
    }
    set packageJson(info) {
        fs.writeFileSync(this.packagePath, JSON.stringify(info, null, "\t"));
    }
    get packageFramerId() {
        const info = this.packageJson;
        if (!info.framer) {
            info.framer = {};
        }
        if (!info.framer.id) {
            info.framer.id = uuid();
            this.packageJson = info;
        }
        return info.framer.id;
    }
    framerPackages() {
        const packages = {};
        const getJSON = (filePath) => {
            try {
                return JSON.parse(fs.readFileSync(filePath).toString());
            }
            catch (error) {
                return;
            }
        };
        const dependencies = Object.keys(this.packageJson.dependencies || {}).concat(Object.keys(this.packageJson.devDependencies || {}));
        for (let dependency of dependencies) {
            let json;
            if (!json) {
                // This is the path for a package that was installed from a registry
                const registryPackagePath = path.join(this.nodeModulesPath, dependency, "package.json");
                json = getJSON(registryPackagePath);
            }
            let dependencyPath;
            if (this.packageJson.dependencies && this.packageJson.dependencies[dependency]) {
                dependencyPath = this.packageJson.dependencies[dependency];
            }
            else if (this.packageJson.devDependencies && this.packageJson.devDependencies[dependency]) {
                dependencyPath = this.packageJson.devDependencies[dependency];
            }
            if (!json) {
                // This is the path for a package that was installed locally
                const localPackagePath = path.join(this.path, dependencyPath, "package.json");
                json = getJSON(localPackagePath);
            }
            if (json && json.framer && json.name) {
                packages[json.name] = json;
                let design;
                if (!design) {
                    const moduleDesignPath = path.join(this.nodeModulesPath, dependency, "design", "document.json");
                    design = getJSON(moduleDesignPath);
                }
                if (!design) {
                    const localDesignPath = path.join(this.path, dependencyPath, "design", "document.json");
                    design = getJSON(localDesignPath);
                }
                if (design) {
                    json.design = design;
                }
            }
        }
        return packages;
    }
    get info() {
        return {
            path: this.path,
        };
    }
    reloadClients() {
        if (this._server) {
            this._server.io.emit("reload");
        }
    }
    serve(port) {
        return __awaiter(this, void 0, void 0, function* () {
            this._server = new server_1.Server(this);
            this.sendScript();
            this.on("script", this.sendScript);
            this.updateServerPackagesListing();
            return this._server.start(port);
        });
    }
    watch() {
        if (this._watcher) {
            this._watcher.stop();
        }
        const watcher = new webpack.Compiler(this, { outputPath: this.buildOutputPath });
        this._watcher = watcher;
        watcher.progress.on("start", () => {
            this.runTask("build", new Promise(resolve => {
                watcher.progress.once("complete", resolve);
            }));
        });
        watcher.watch(() => {
            this._lastBuildOutput = watcher.buildOutput;
            this.emit("script");
        });
    }
    invalidateCompilerWatch() {
        if (!this._watcher) {
            return;
        }
        this._watcher.invalidate();
    }
    watchAssets() {
        // Should be only called once
        if (this._assetWatcher)
            return;
        const assets = [
            `${this.config.paths.metadata}/icon.png`,
            `${this.config.paths.metadata}/artwork.png`,
            "package.json",
            "README.md",
        ];
        // The "*" and "metadata/*" are needed to pick up newly created files
        this._assetWatcher = new gaze.Gaze(["*", "metadata/*"], { debounceDelay: 100, cwd: this.path });
        this._assetWatcher.on("all", (event, filepath) => {
            const relativePath = filepath.replace(`${this.path}/`, "");
            // Because of the globbing it picks up other changed files too
            // This only sends events when relevant files are changed
            if (assets.includes(relativePath) && this._server) {
                const url = `${this._server.state.url}/${relativePath}`;
                this._server.io.emit("filechange", { relativePath, url, event });
            }
        });
        this._assetWatcher.on("error", (error) => {
            // ignore ENOENT errors, they are almost all bugs in gaze.js and it's use of fs.watch
            if (error && error.code === "ENOENT")
                return;
            // in Windows, we seem to be getting { code:EPERM, errno:-4048 }
            if (os.platform() == "win32" && error && error.code === "EPERM")
                return;
            // emitting other errors ourselves, will report and exit the nodejs process
            this.emit("error", error);
        });
    }
    prepareForPublish(packageName, displayName) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.updatePackageJson(packageJson => {
                if (packageJson.main === "build/index.js") {
                    // Override "main" to ensure that legacy projects are updated.
                    packageJson.main = "dist/index.js";
                }
                if (packageName) {
                    packageJson.name = packageName;
                }
                if (displayName) {
                    if (!packageJson.framer)
                        packageJson.framer = {};
                    packageJson.framer.displayName = displayName;
                }
                return packageJson;
            });
        });
    }
    updatePackageJson(updater) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const packageInfoData = yield fs.readFile(this.packagePath);
                let packageInfo = JSON.parse(packageInfoData.toString());
                packageInfo = updater(packageInfo);
                const newPackageInfoData = Buffer.from(JSON.stringify(packageInfo, null, "\t"));
                if (packageInfoData.equals(newPackageInfoData)) {
                    // Don't write to the file if there's no change to avoid watch triggering.
                    return;
                }
                yield fs.writeFile(this.packagePath, newPackageInfoData);
            }
            catch (error) {
                throw Error(`Error updating package.json ${error}`);
            }
        });
    }
    // Link the node_modules directory to a symlink in the cache, if needed
    _fixNodeModulesPath() {
        if (!fs.pathExistsSync(this.nodeModulesPath)) {
            // If it’s an symlink that points to a directory that does not exist, we end up here,
            // to be sure, try to remove the broken symlink
            try {
                fs.removeSync(this.nodeModulesPath);
            }
            finally {
            }
        }
        // Retrieve the packageFramerId so it gets added to the package.json if it does not exist
        // TODO: Remove this in the future. The ID is not actually used anymore
        // This is here mostly to mimic old behavior that is being relied on by the file format
        this.packageFramerId;
    }
    updateServerPackagesListing() {
        if (!this._server)
            return;
        const state = {};
        const dependencies = this.packageJson.dependencies;
        if (dependencies) {
            state.packages = Object.keys(dependencies);
            this.updatePackagesInfo(state.packages);
        }
        const devDependencies = this.packageJson.devDependencies;
        // TODO: Add devDependencies also to packages? (because they are also installed)
        if (devDependencies) {
            const version = devDependencies["framer"];
            if (typeof version === "string") {
                state.libraryVersion = version.startsWith("^") ? version.substring(1) : version;
            }
        }
        this._server.setState(state);
    }
    updatePackagesInfo(packages) {
        const server = this._server;
        if (!server)
            return;
        const unknownPackages = packages.filter(packageId => !server.hasPackageInfo(packageId));
        if (unknownPackages.length === 0)
            return;
        const url = `${this.api}/store/meta/get-many`;
        const request = new node_fetch_1.Request(url, {
            headers: { Authorization: `Token ${this.token}`, ["Content-Type"]: "application/json" },
            body: JSON.stringify({ packageNames: unknownPackages }),
            method: "POST",
        });
        node_fetch_1.default(request)
            .then(response => response.json())
            .then((json) => server.addPackageInfo(json))
            .catch(() => console.log("Could not fetch package info"));
    }
}
exports.Project = Project;
function copyResourceFiles(container, map) {
    return new Promise(resolve => {
        let scheduled = 0;
        function maybeDone() {
            if (scheduled === 0) {
                resolve();
            }
        }
        for (const key in map) {
            const source = path.join(__dirname, key);
            const target = path.join(container, map[key]);
            let statSource;
            try {
                statSource = fs.statSync(source);
            }
            catch (e) {
                console.error("unable to find resource file:", key);
                continue;
            }
            try {
                const statTarget = fs.statSync(target);
                if (+statSource.mtime === +statTarget.mtime && statSource.size === statTarget.size) {
                    continue;
                }
            }
            catch (_a) { }
            scheduled += 1;
            fs.copy(source, target, { preserveTimestamps: true }, err => {
                if (err)
                    console.error(err);
                scheduled -= 1;
                maybeDone();
            });
        }
        maybeDone();
    });
}
